/**
 * Weex driver
 */
var STYLE = 'style';
var ID = 'id';
var TEXT = 'text';
var CHILDREN = 'children';
var EVENT_PREFIX_REGEXP = /^on[A-Z]/;
var ARIA_PREFIX_REGEXP = /^aria-/;
var EMPTY = '';
var nodeMaps = {};
/* global __weex_document__ */

var document = typeof __weex_document__ === 'object' ? __weex_document__ : typeof document === 'object' ? document : null;
function getElementById(id) {
  return nodeMaps[id];
}
function createBody(type, props) {
  if (document.body) {
    return document.body;
  }

  var documentElement = document.documentElement;
  var body = document.createBody(type, props);
  documentElement.appendChild(body);
  return body;
}
function createComment(content) {
  return document.createComment(content);
}
function createEmpty() {
  return this.createComment(EMPTY);
}
function createText(text) {
  return createElement(TEXT, {
    value: text
  });
}
function updateText(node, content) {
  setAttribute(node, 'value', content);
}
function createElement(type, props) {
  var node = document.createElement(type, {
    style: props[STYLE]
  });

  for (var prop in props) {
    var value = props[prop];

    if (prop === CHILDREN) {
      continue;
    }

    if (value != null) {
      if (prop === STYLE) {
        continue;
      } else if (EVENT_PREFIX_REGEXP.test(prop)) {
        var eventName = prop.slice(2).toLowerCase();
        addEventListener(node, eventName, value, props);
      } else {
        setAttribute(node, prop, value);
      }
    }
  }

  return node;
}
function appendChild(node, parent) {
  return parent.appendChild(node);
}
function removeChild(node, parent) {
  parent = parent || node.parentNode;
  var id = node.attr && node.attr[ID];

  if (id != null) {
    nodeMaps[id] = null;
  }

  return parent.removeChild(node);
}
function replaceChild(newChild, oldChild, parent) {
  parent = parent || oldChild.parentNode;
  var previousSibling = oldChild.previousSibling;
  var nextSibling = oldChild.nextSibling;
  removeChild(oldChild, parent);

  if (previousSibling) {
    insertAfter(newChild, previousSibling, parent);
  } else if (nextSibling) {
    insertBefore(newChild, nextSibling, parent);
  } else {
    appendChild(newChild, parent);
  }
}
function insertAfter(node, after, parent) {
  parent = parent || after.parentNode;
  return parent.insertAfter(node, after);
}
function insertBefore(node, before, parent) {
  parent = parent || before.parentNode;
  return parent.insertBefore(node, before);
}
function addEventListener(node, eventName, eventHandler, props) {
  // https://github.com/apache/incubator-weex/blob/master/runtime/vdom/Element.js#L421
  var params = props[eventName + 'EventParams'];
  return node.addEvent(eventName, eventHandler, params);
}
function removeEventListener(node, eventName, eventHandler) {
  return node.removeEvent(eventName, eventHandler);
}
function removeAttribute(node, propKey, propValue) {
  if (propKey == ID) {
    nodeMaps[propValue] = null;
  } // Weex native will crash when pass null value


  return node.setAttr(propKey, undefined, false);
}
function setAttribute(node, propKey, propValue) {
  if (propKey == ID) {
    nodeMaps[propValue] = node;
  } // Weex only support `ariaLabel` format, convert `aria-label` format to camelcase


  if (ARIA_PREFIX_REGEXP.test(propKey)) {
    propKey = propKey.replace(/\-(\w)/, function (m, p) {
      return p.toUpperCase();
    });
  }

  return node.setAttr(propKey, propValue, false);
}
function setStyle(node, style) {
  node.setStyles(style);
}
function beforeRender() {
  // Turn off batched updates
  document.open();
}
function afterRender() {
  if (document.listener && document.listener.createFinish) {
    document.listener.createFinish();
  } // Turn on batched updates


  document.close();
}

export { getElementById, createBody, createComment, createEmpty, createText, updateText, createElement, appendChild, removeChild, replaceChild, insertAfter, insertBefore, addEventListener, removeEventListener, removeAttribute, setAttribute, setStyle, beforeRender, afterRender };
//# sourceMappingURL=driver-weex.mjs.map
